/*
 * kernel/power/suspend.h
 *
 * Copyright (C) 2004-2006 Nigel Cunningham <nigel@suspend2.net>
 * Copyright (C) 2006 Red Hat, inc.
 *
 * This file is released under the GPLv2.
 *
 * It contains declarations used throughout swsusp.
 *
 */

#ifndef KERNEL_POWER_SUSPEND_H
#define KERNEL_POWER_SUSPEND_H

#include <linux/delay.h>
#include <linux/bootmem.h>
#include <linux/suspend.h>
#include <asm/setup.h>

#define SUSPEND_CORE_VERSION "2.2.9.3"

extern int save_image_part1(void);
extern int suspend_atomic_restore(void);
extern int __suspend2_try_suspend(int had_pmsem);

#define KB(x) ((x) << (PAGE_SHIFT - 10))
#define MB(x) ((x) >> (20 - PAGE_SHIFT))

extern int suspend_start_anything(int starting_cycle);
extern void suspend_finish_anything(int finishing_cycle);

extern int __suspend_post_context_save(void);

struct suspend2_core_fns {
	int (*post_context_save)(void);
	unsigned long (*get_nonconflicting_page)(void);
	int (*try_suspend)(int have_pmsem);
	void (*try_resume)(void);
};

extern struct suspend2_core_fns *s2_core_fns;

#define set_debug_state(bit) (test_and_set_bit(bit, &suspend_debug_state))
#define clear_debug_state(bit) (test_and_clear_bit(bit, &suspend_debug_state))

#define set_result_state(bit) (test_and_set_bit(bit, &suspend_result))
#define clear_result_state(bit) (test_and_clear_bit(bit, &suspend_result))

enum {
	SUSPEND_ABORTED,
	SUSPEND_ABORT_REQUESTED,
	SUSPEND_NOSTORAGE_AVAILABLE,
	SUSPEND_INSUFFICIENT_STORAGE,
	SUSPEND_FREEZING_FAILED,
	SUSPEND_UNEXPECTED_ALLOC,
	SUSPEND_KEPT_IMAGE,
	SUSPEND_WOULD_EAT_MEMORY,
	SUSPEND_UNABLE_TO_FREE_ENOUGH_MEMORY,
	SUSPEND_ENCRYPTION_SETUP_FAILED,
	SUSPEND_PM_SEM,
	SUSPEND_DEVICE_REFUSED,
	SUSPEND_EXTRA_PAGES_ALLOW_TOO_SMALL,
};

extern unsigned int nr_suspends;
extern char resume2_file[256];
extern char poweroff_resume2[256];

extern void copyback_post(void);
extern int suspend2_suspend(void);
extern int extra_pd1_pages_used;

enum {
	STEP_SUSPEND_PREPARE_IMAGE,
	STEP_SUSPEND_SAVE_IMAGE,
	STEP_SUSPEND_POWERDOWN,
	STEP_RESUME_CAN_RESUME,
	STEP_RESUME_LOAD_PS1,
	STEP_RESUME_DO_RESTORE,
	STEP_RESUME_READ_PS2,
	STEP_RESUME_GO,
	STEP_RESUME_ALT_IMAGE,
};

#endif
