#include <linux/module.h>
#include <linux/resume-trace.h>
#include <linux/syscalls.h>
#include <linux/kernel.h>
#include <linux/swap.h>
#include <linux/syscalls.h>
#include <linux/bio.h>
#include <linux/root_dev.h>
#include <linux/freezer.h>
#include <linux/reboot.h>
#include <linux/writeback.h>
#include <linux/tty.h>
#include <linux/crypto.h>
#include <linux/cpu.h>
#include "io.h"
#include "suspend.h"
#include "extent.h"
#include "block_io.h"
#include "netlink.h"
#include "prepare_image.h"
#include "ui.h"
#include "sysfs.h"
#include "pagedir.h"
#include "modules.h"

#ifdef CONFIG_SUSPEND2_CORE_EXPORTS
#ifdef CONFIG_SOFTWARE_SUSPEND
EXPORT_SYMBOL(resume_file);
#endif

EXPORT_SYMBOL(max_pfn);
EXPORT_SYMBOL(free_dyn_pageflags);
EXPORT_SYMBOL(clear_dynpageflag);
EXPORT_SYMBOL(test_dynpageflag);
EXPORT_SYMBOL(set_dynpageflag);
EXPORT_SYMBOL(get_next_bit_on);
EXPORT_SYMBOL(allocate_dyn_pageflags);
EXPORT_SYMBOL(clear_dyn_pageflags);

#ifdef CONFIG_X86_64
EXPORT_SYMBOL(restore_processor_state);
EXPORT_SYMBOL(save_processor_state);
#endif

EXPORT_SYMBOL(kernel_shutdown_prepare);
EXPORT_SYMBOL(drop_pagecache);
EXPORT_SYMBOL(restore_pblist);
EXPORT_SYMBOL(pm_mutex);
EXPORT_SYMBOL(console_printk);
EXPORT_SYMBOL(pm_restore_console);
EXPORT_SYMBOL(super_blocks);
EXPORT_SYMBOL(next_zone);

EXPORT_SYMBOL(freeze_processes);
EXPORT_SYMBOL(thaw_processes);
EXPORT_SYMBOL(thaw_kernel_threads);
EXPORT_SYMBOL(shrink_all_memory);
EXPORT_SYMBOL(saveable_page);
EXPORT_SYMBOL(swsusp_arch_suspend);
EXPORT_SYMBOL(swsusp_arch_resume);
EXPORT_SYMBOL(pm_ops);
EXPORT_SYMBOL(pm_prepare_console);
EXPORT_SYMBOL(suspend2_follow_page);
EXPORT_SYMBOL(machine_halt);
EXPORT_SYMBOL(block_dump);
EXPORT_SYMBOL(unlink_lru_lists);
EXPORT_SYMBOL(relink_lru_lists);
EXPORT_SYMBOL(kmsg_redirect);
EXPORT_SYMBOL(power_subsys);
EXPORT_SYMBOL(machine_power_off);
EXPORT_SYMBOL(suspend_enter);
EXPORT_SYMBOL(first_online_pgdat);
EXPORT_SYMBOL(machine_restart);
EXPORT_SYMBOL(saved_command_line);
EXPORT_SYMBOL(tasklist_lock);
#ifdef CONFIG_SUSPEND_SMP
EXPORT_SYMBOL(disable_nonboot_cpus);
EXPORT_SYMBOL(enable_nonboot_cpus);
#endif
#endif

#ifdef CONFIG_SUSPEND2_SWAP_EXPORTS	/* Suspend swap specific */
EXPORT_SYMBOL(sys_swapon);
EXPORT_SYMBOL(sys_swapoff);
EXPORT_SYMBOL(si_swapinfo);
EXPORT_SYMBOL(map_swap_page);
EXPORT_SYMBOL(get_swap_page);
EXPORT_SYMBOL(swap_free);
EXPORT_SYMBOL(get_swap_info_struct);
#endif

#ifdef CONFIG_SUSPEND2_FILE_EXPORTS
/* Suspend_file specific */
extern char * __initdata root_device_name;

EXPORT_SYMBOL(ROOT_DEV);
EXPORT_SYMBOL(root_device_name);
EXPORT_SYMBOL(sys_unlink);
EXPORT_SYMBOL(sys_mknod);
EXPORT_SYMBOL(name_to_dev_t);
#endif

/* Swap or file */
#if defined(CONFIG_SUSPEND2_FILE_EXPORTS) || defined(CONFIG_SUSPEND2_SWAP_EXPORTS)
EXPORT_SYMBOL(bio_set_pages_dirty);
#endif

#if defined(CONFIG_SUSPEND2_EXPORTS) || defined(CONFIG_SUSPEND2_CORE_EXPORTS)
EXPORT_SYMBOL(snprintf_used);
#endif
struct suspend2_core_fns *s2_core_fns;
EXPORT_SYMBOL(s2_core_fns);

unsigned long suspend_get_nonconflicting_page(void)
{
	return s2_core_fns->get_nonconflicting_page();
}

int suspend_post_context_save(void)
{
	return s2_core_fns->post_context_save();
}

int suspend2_try_suspend(int have_pmsem)
{
	if (!s2_core_fns)
		return -ENODEV;

	return s2_core_fns->try_suspend(have_pmsem);
}

void suspend2_try_resume(void)
{
	if (s2_core_fns)
		s2_core_fns->try_resume();
}

unsigned long suspend_compress_bytes_in, suspend_compress_bytes_out;
EXPORT_SYMBOL(suspend_compress_bytes_in);
EXPORT_SYMBOL(suspend_compress_bytes_out);

#ifdef CONFIG_SUSPEND2_REPLACE_SWSUSP
unsigned long suspend_action = (1 << SUSPEND_REPLACE_SWSUSP) | (1 << SUSPEND_PAGESET2_FULL);
#else
unsigned long suspend_action = 1 << SUSPEND_PAGESET2_FULL;
#endif
EXPORT_SYMBOL(suspend_action);

unsigned long suspend_state = ((1 << SUSPEND_BOOT_TIME) |
		(1 << SUSPEND_RESUME_NOT_DONE) | (1 << SUSPEND_IGNORE_LOGLEVEL));
EXPORT_SYMBOL(suspend_state);

/* The number of suspends we have started (some may have been cancelled) */
unsigned int nr_suspends;
EXPORT_SYMBOL(nr_suspends);

char resume2_file[256] = CONFIG_SUSPEND2_DEFAULT_RESUME2;
EXPORT_SYMBOL(resume2_file);

int suspend2_running = 0;
EXPORT_SYMBOL(suspend2_running);

int suspend2_in_suspend __nosavedata;
EXPORT_SYMBOL(suspend2_in_suspend);

unsigned long suspend2_nosave_state1 __nosavedata = 0;
unsigned long suspend2_nosave_state2 __nosavedata = 0;
int suspend2_nosave_state3 __nosavedata = 0;
int suspend2_nosave_io_speed[2][2] __nosavedata;
__nosavedata char suspend2_nosave_commandline[COMMAND_LINE_SIZE];

dyn_pageflags_t __nosavedata suspend2_nosave_origmap;
dyn_pageflags_t __nosavedata suspend2_nosave_copymap;

#ifdef CONFIG_HIGHMEM
__nosavedata struct zone_data *suspend2_nosave_zone_list;
__nosavedata unsigned long suspend2_nosave_max_pfn;
#endif

#ifdef CONFIG_SUSPEND2_CORE_EXPORTS
#ifdef CONFIG_HIGHMEM
EXPORT_SYMBOL(suspend2_nosave_max_pfn);
EXPORT_SYMBOL(suspend2_nosave_zone_list);
EXPORT_SYMBOL(nr_free_highpages);
EXPORT_SYMBOL(saveable_highmem_page);
#endif

#ifndef CONFIG_COMPAT
EXPORT_SYMBOL(sys_ioctl);
#endif

EXPORT_SYMBOL(suspend2_nosave_origmap);
EXPORT_SYMBOL(suspend2_nosave_copymap);
EXPORT_SYMBOL(suspend2_nosave_state1);
EXPORT_SYMBOL(suspend2_nosave_state2);
EXPORT_SYMBOL(suspend2_nosave_state3);
EXPORT_SYMBOL(suspend2_nosave_io_speed);
EXPORT_SYMBOL(suspend2_nosave_commandline);
#ifdef CONFIG_PM_TRACE
EXPORT_SYMBOL(pm_trace_enabled);
#endif
#endif

/* --  Commandline Parameter Handling ---
 *
 * Resume setup: obtain the storage device.
 */
static int __init resume2_setup(char *str)
{
	if (!*str)
		return 0;
	
	strncpy(resume2_file, str, 255);
	return 0;
}

/*
 * Allow the user to specify that we should ignore any image found and
 * invalidate the image if necesssary. This is equivalent to running
 * the task queue and a sync and then turning off the power. The same
 * precautions should be taken: fsck if you're not journalled.
 */
static int __init noresume2_setup(char *str)
{
	set_suspend_state(SUSPEND_NORESUME_SPECIFIED);
	return 0;
}

static int __init suspend_retry_resume_setup(char *str)
{
	set_suspend_state(SUSPEND_RETRY_RESUME);
	return 0;
}

#ifndef CONFIG_SOFTWARE_SUSPEND
static int __init resume_setup(char *str)
{
	if (!*str)
		return 0;
	
	strncpy(resume2_file, str, 255);
	return 0;
}

static int __init noresume_setup(char *str)
{
	set_suspend_state(SUSPEND_NORESUME_SPECIFIED);
	return 0;
}
__setup("noresume", noresume_setup);
__setup("resume=", resume_setup);
#endif

__setup("noresume2", noresume2_setup);
__setup("resume2=", resume2_setup);
__setup("suspend_retry_resume", suspend_retry_resume_setup);

